-- Align a user-defined plane with either XY, XZ or YZ plane.
-- The resulting transformation is applied to all meshes in the scene.

-- author: Dusan Pavlicek, pavlicd@volny.cz, September 2007


fn GetSelectedVertices = (
	selectedVertices = #()
	selObjects = selection as array
	for i = 1 to selObjects.count do (
		object = selObjects[i]
		if classof object == Editable_Mesh then (
			selVertexIDs = getVertSelection object as array
			for j = 1 to selVertexIDs.count do (
				vertex = getVert object selVertexIDs[j]
				append selectedVertices vertex
			)			
		)
	)
	return selectedVertices
)


fn CalcAlignmentQuat plane axis = (
	axis = normalize axis
	planeU = plane[2] - plane[1]
	planeV = plane[3] - plane[1]
	planeNormal = normalize (cross planeU planeV)
	rotAngle = acos(dot planeNormal axis)
	if abs rotAngle < 0.0001 then (
		return undefined
	)
	rotAxis = normalize (cross planeNormal axis)
	return quat rotAngle rotAxis
)


fn AlignMeshes alignQuat = (
	print alignQuat
	for object in rootNode.children do (
		if classof object == Editable_Mesh do (
			rotate object (angleaxis alignQuat.angle alignQuat.axis)
		)
	)
)


rollout RolloutMain "" (
	label label1 "Select 3 vertices to define"
	label label2 "a plane you want to align."
	label label3 ""
	label label4 "Then choose the desired alignment"
	label label5 "and press the button."
	
	button buttonAlignXY "Align plane with XY"
	button buttonAlignXZ "Align plane with XZ"
	button buttonAlignYZ "Align plane with YZ"

	on buttonAlignXY pressed do (
		plane = GetSelectedVertices()
		if plane.count != 3 then (
			MessageBox("Select exactly 3 vertices please.")
		)
		else (
			alignQuat = CalcAlignmentQuat plane (point3 0 0 1)
			if alignQuat != undefined then (
				AlignMeshes alignQuat
			)
		)
	)
	
	on buttonAlignXZ pressed do (
		plane = GetSelectedVertices()
		if plane.count != 3 then (
			MessageBox("Select exactly 3 vertices please.")
		)
		else (
			alignQuat = CalcAlignmentQuat plane (point3 0 1 0)
			if alignQuat != undefined then (
				AlignMeshes alignQuat
			)
		)
	)
	
	on buttonAlignYZ pressed do (
		plane = GetSelectedVertices()
		if plane.count != 3 then (
			MessageBox("Select exactly 3 vertices please.")
		)
		else (
			alignQuat = CalcAlignmentQuat plane (point3 1 0 0)
			if alignQuat != undefined then (
				AlignMeshes alignQuat
			)
		)
	)
)


rollout RolloutAbout "About" (
	label label1 "Dusan Pavlicek"
	label label2 "pavlicd@volny.cz"
	label label3 "September 2007"
)


if MyFloater != undefined do (
	closerolloutfloater MyFloater
)
MyFloater = newRolloutFloater "Align Plane" 250 350
addRollout RolloutMain MyFloater
addRollout RolloutAbout MyFloater
